import Foundation

precedencegroup RegularPrecedence {
    
    lowerThan: AdditionPrecedence       // 优先级, 比加法运算低
    associativity: none                 // 结合方向:left, right or none
    assignment: false                   // true=赋值运算符,false=非赋值运算符
}

fileprivate var _predicate: NSPredicate!
fileprivate var _regular: String = ""

infix operator ~=: RegularPrecedence
func ~=(origin: String, regular: String) -> Bool {
    
    if _regular != regular {
        
        _regular = regular
        _predicate = NSPredicate(format: "SELF MATCHES %@", _regular)
    }
    return _predicate.evaluate(with: origin)
}

extension String {
    
    public func match(_ regular: String) -> Bool {
        
        return self ~= regular
    }
    
    public func trim(_ occurrences: String = " ") -> String {
        
        return replacingOccurrences(of: occurrences, with: "")
    }
    
    public func split(_ characterSet: String) -> [String] {
        
        return components(separatedBy: CharacterSet(charactersIn: characterSet))
    }
    
    public func upperFrist() -> String {
        
        var value = self
        let character = value.remove(at: value.startIndex)
        return "\(character)".uppercased() + value
    }
    
    public func lowerFrist() -> String {
        
        var value = self
        let character = value.remove(at: value.startIndex)
        return "\(character)".lowercased() + value
    }
    
    public func union() -> String {
        
        guard contains("-") else {
            
            return self
        }
        
        var result: String = ""
        let elements = split(separator: "-").compactMap{ "\($0)" }
        elements.forEach { (item) in
            
            result = result + item.upperFrist()
        }
        
        return result
    }
}

